@@ -2,18 +2,19 @@ use core::cmp::min;
 #[cfg(feature = "async")]
 use core::task::Waker;
 
-use crate::{Error, Result};
+use crate::iface::Context;
 use crate::phy::ChecksumCapabilities;
-use crate::socket::{Socket, SocketMeta, SocketHandle, PollAt};
-use crate::storage::{PacketBuffer, PacketMetadata};
+use crate::socket::PollAt;
 #[cfg(feature = "async")]
 use crate::socket::WakerRegistration;
+use crate::storage::{PacketBuffer, PacketMetadata};
+use crate::{Error, Result};
 
-use crate::wire::{IpVersion, IpRepr, IpProtocol};
+use crate::wire::{IpProtocol, IpRepr, IpVersion};
 #[cfg(feature = "proto-ipv4")]
-use crate::wire::{Ipv4Repr, Ipv4Packet};
+use crate::wire::{Ipv4Packet, Ipv4Repr};
 #[cfg(feature = "proto-ipv6")]
-use crate::wire::{Ipv6Repr, Ipv6Packet};
+use crate::wire::{Ipv6Packet, Ipv6Repr};
 
 /// A UDP packet metadata.
 pub type RawPacketMetadata = PacketMetadata<()>;
@@ -27,11 +28,10 @@ pub type RawSocketBuffer<'a> = PacketBuffer<'a, ()>;
 /// transmit and receive packet buffers.
 #[derive(Debug)]
 pub struct RawSocket<'a> {
-    pub(crate) meta: SocketMeta,
-    ip_version:  IpVersion,
+    ip_version: IpVersion,
     ip_protocol: IpProtocol,
-    rx_buffer:   RawSocketBuffer<'a>,
-    tx_buffer:   RawSocketBuffer<'a>,
+    rx_buffer: RawSocketBuffer<'a>,
+    tx_buffer: RawSocketBuffer<'a>,
     #[cfg(feature = "async")]
     rx_waker: WakerRegistration,
     #[cfg(feature = "async")]
@@ -41,11 +41,13 @@ pub struct RawSocket<'a> {
 impl<'a> RawSocket<'a> {
     /// Create a raw IP socket bound to the given IP version and datagram protocol,
     /// with the given buffers.
-    pub fn new(ip_version: IpVersion, ip_protocol: IpProtocol,
-               rx_buffer: RawSocketBuffer<'a>,
-               tx_buffer: RawSocketBuffer<'a>) -> RawSocket<'a> {
+    pub fn new(
+        ip_version: IpVersion,
+        ip_protocol: IpProtocol,
+        rx_buffer: RawSocketBuffer<'a>,
+        tx_buffer: RawSocketBuffer<'a>,
+    ) -> RawSocket<'a> {
         RawSocket {
-            meta: SocketMeta::default(),
             ip_version,
             ip_protocol,
             rx_buffer,
@@ -61,12 +63,12 @@ impl<'a> RawSocket<'a> {
     ///
     /// The waker is woken on state changes that might affect the return value
     /// of `recv` method calls, such as receiving data, or the socket closing.
-    /// 
+    ///
     /// Notes:
     ///
     /// - Only one waker can be registered at a time. If another waker was previously registered,
     ///   it is overwritten and will no longer be woken.
-    /// - The Waker is woken only once. Once woken, you must register it again to receive more wakes. 
+    /// - The Waker is woken only once. Once woken, you must register it again to receive more wakes.
     /// - "Spurious wakes" are allowed: a wake doesn't guarantee the result of `recv` has
     ///   necessarily changed.
     #[cfg(feature = "async")]
@@ -79,12 +81,12 @@ impl<'a> RawSocket<'a> {
     /// The waker is woken on state changes that might affect the return value
     /// of `send` method calls, such as space becoming available in the transmit
     /// buffer, or the socket closing.
-    /// 
+    ///
     /// Notes:
     ///
     /// - Only one waker can be registered at a time. If another waker was previously registered,
     ///   it is overwritten and will no longer be woken.
-    /// - The Waker is woken only once. Once woken, you must register it again to receive more wakes. 
+    /// - The Waker is woken only once. Once woken, you must register it again to receive more wakes.
     /// - "Spurious wakes" are allowed: a wake doesn't guarantee the result of `send` has
     ///   necessarily changed.
     #[cfg(feature = "async")]
@@ -92,12 +94,6 @@ impl<'a> RawSocket<'a> {
         self.tx_waker.register(waker)
     }
 
-    /// Return the socket handle.
-    #[inline]
-    pub fn handle(&self) -> SocketHandle {
-        self.meta.handle
-    }
-
     /// Return the IP version the socket is bound to.
     #[inline]
     pub fn ip_version(&self) -> IpVersion {
@@ -160,9 +156,12 @@ impl<'a> RawSocket<'a> {
     pub fn send(&mut self, size: usize) -> Result<&mut [u8]> {
         let packet_buf = self.tx_buffer.enqueue(size, ())?;
 
-        net_trace!("{}:{}:{}: buffer to send {} octets",
-                   self.meta.handle, self.ip_version, self.ip_protocol,
-                   packet_buf.len());
+        net_trace!(
+            "raw:{}:{}: buffer to send {} octets",
+            self.ip_version,
+            self.ip_protocol,
+            packet_buf.len()
+        );
         Ok(packet_buf)
     }
 
@@ -183,9 +182,12 @@ impl<'a> RawSocket<'a> {
     pub fn recv(&mut self) -> Result<&[u8]> {
         let ((), packet_buf) = self.rx_buffer.dequeue()?;
 
-        net_trace!("{}:{}:{}: receive {} buffered octets",
-                   self.meta.handle, self.ip_version, self.ip_protocol,
-                   packet_buf.len());
+        net_trace!(
+            "raw:{}:{}: receive {} buffered octets",
+            self.ip_version,
+            self.ip_protocol,
+            packet_buf.len()
+        );
         Ok(packet_buf)
     }
 
@@ -200,25 +202,36 @@ impl<'a> RawSocket<'a> {
     }
 
     pub(crate) fn accepts(&self, ip_repr: &IpRepr) -> bool {
-        if ip_repr.version() != self.ip_version { return false }
-        if ip_repr.protocol() != self.ip_protocol { return false }
+        if ip_repr.version() != self.ip_version {
+            return false;
+        }
+        if ip_repr.protocol() != self.ip_protocol {
+            return false;
+        }
 
         true
     }
 
-    pub(crate) fn process(&mut self, ip_repr: &IpRepr, payload: &[u8],
-                          checksum_caps: &ChecksumCapabilities) -> Result<()> {
+    pub(crate) fn process(
+        &mut self,
+        cx: &mut Context,
+        ip_repr: &IpRepr,
+        payload: &[u8],
+    ) -> Result<()> {
         debug_assert!(self.accepts(ip_repr));
 
         let header_len = ip_repr.buffer_len();
-        let total_len  = header_len + payload.len();
+        let total_len = header_len + payload.len();
         let packet_buf = self.rx_buffer.enqueue(total_len, ())?;
-        ip_repr.emit(&mut packet_buf[..header_len], &checksum_caps);
+        ip_repr.emit(&mut packet_buf[..header_len], &cx.checksum_caps());
         packet_buf[header_len..].copy_from_slice(payload);
 
-        net_trace!("{}:{}:{}: receiving {} octets",
-                   self.meta.handle, self.ip_version, self.ip_protocol,
-                   packet_buf.len());
+        net_trace!(
+            "raw:{}:{}: receiving {} octets",
+            self.ip_version,
+            self.ip_protocol,
+            packet_buf.len()
+        );
 
         #[cfg(feature = "async")]
         self.rx_waker.wake();
@@ -226,16 +239,22 @@ impl<'a> RawSocket<'a> {
         Ok(())
     }
 
-    pub(crate) fn dispatch<F>(&mut self, checksum_caps: &ChecksumCapabilities, emit: F) ->
-                             Result<()>
-            where F: FnOnce((IpRepr, &[u8])) -> Result<()> {
-        fn prepare<'a>(protocol: IpProtocol, buffer: &'a mut [u8],
-                   _checksum_caps: &ChecksumCapabilities) -> Result<(IpRepr, &'a [u8])> {
+    pub(crate) fn dispatch<F>(&mut self, cx: &mut Context, emit: F) -> Result<()>
+    where
+        F: FnOnce(&mut Context, (IpRepr, &[u8])) -> Result<()>,
+    {
+        fn prepare<'a>(
+            protocol: IpProtocol,
+            buffer: &'a mut [u8],
+            _checksum_caps: &ChecksumCapabilities,
+        ) -> Result<(IpRepr, &'a [u8])> {
             match IpVersion::of_packet(buffer)? {
                 #[cfg(feature = "proto-ipv4")]
                 IpVersion::Ipv4 => {
                     let mut packet = Ipv4Packet::new_checked(buffer)?;
-                    if packet.protocol() != protocol { return Err(Error::Unaddressable) }
+                    if packet.protocol() != protocol {
+                        return Err(Error::Unaddressable);
+                    }
                     if _checksum_caps.ipv4.tx() {
                         packet.fill_checksum();
                     } else {
@@ -251,7 +270,9 @@ impl<'a> RawSocket<'a> {
                 #[cfg(feature = "proto-ipv6")]
                 IpVersion::Ipv6 => {
                     let packet = Ipv6Packet::new_checked(buffer)?;
-                    if packet.next_header() != protocol { return Err(Error::Unaddressable) }
+                    if packet.next_header() != protocol {
+                        return Err(Error::Unaddressable);
+                    }
                     let packet = Ipv6Packet::new_unchecked(&*packet.into_inner());
                     let ipv6_repr = Ipv6Repr::parse(&packet)?;
                     Ok((IpRepr::Ipv6(ipv6_repr), packet.payload()))
@@ -260,21 +281,26 @@ impl<'a> RawSocket<'a> {
             }
         }
 
-        let handle      = self.meta.handle;
         let ip_protocol = self.ip_protocol;
-        let ip_version  = self.ip_version;
+        let ip_version = self.ip_version;
         self.tx_buffer.dequeue_with(|&mut (), packet_buf| {
-            match prepare(ip_protocol, packet_buf, &checksum_caps) {
+            match prepare(ip_protocol, packet_buf, &cx.checksum_caps()) {
                 Ok((ip_repr, raw_packet)) => {
-                    net_trace!("{}:{}:{}: sending {} octets",
-                               handle, ip_version, ip_protocol,
-                               ip_repr.buffer_len() + raw_packet.len());
-                    emit((ip_repr, raw_packet))
+                    net_trace!(
+                        "raw:{}:{}: sending {} octets",
+                        ip_version,
+                        ip_protocol,
+                        ip_repr.buffer_len() + raw_packet.len()
+                    );
+                    emit(cx, (ip_repr, raw_packet))
                 }
                 Err(error) => {
-                    net_debug!("{}:{}:{}: dropping outgoing packet ({})",
-                               handle, ip_version, ip_protocol,
-                               error);
+                    net_debug!(
+                        "raw:{}:{}: dropping outgoing packet ({})",
+                        ip_version,
+                        ip_protocol,
+                        error
+                    );
                     // Return Ok(()) so the packet is dequeued.
                     Ok(())
                 }
@@ -287,7 +313,7 @@ impl<'a> RawSocket<'a> {
         Ok(())
     }
 
-    pub(crate) fn poll_at(&self) -> PollAt {
+    pub(crate) fn poll_at(&self, _cx: &mut Context) -> PollAt {
         if self.tx_buffer.is_empty() {
             PollAt::Ingress
         } else {
@@ -296,34 +322,36 @@ impl<'a> RawSocket<'a> {
     }
 }
 
-impl<'a> Into<Socket<'a>> for RawSocket<'a> {
-    fn into(self) -> Socket<'a> {
-        Socket::Raw(self)
-    }
-}
-
 #[cfg(test)]
 mod test {
+    use super::*;
     use crate::wire::IpRepr;
     #[cfg(feature = "proto-ipv4")]
     use crate::wire::{Ipv4Address, Ipv4Repr};
     #[cfg(feature = "proto-ipv6")]
     use crate::wire::{Ipv6Address, Ipv6Repr};
-    use super::*;
 
     fn buffer(packets: usize) -> RawSocketBuffer<'static> {
-        RawSocketBuffer::new(vec![RawPacketMetadata::EMPTY; packets], vec![0; 48 * packets])
+        RawSocketBuffer::new(
+            vec![RawPacketMetadata::EMPTY; packets],
+            vec![0; 48 * packets],
+        )
     }
 
     #[cfg(feature = "proto-ipv4")]
     mod ipv4_locals {
         use super::*;
 
-        pub fn socket(rx_buffer: RawSocketBuffer<'static>,
-                      tx_buffer: RawSocketBuffer<'static>)
-                     -> RawSocket<'static> {
-            RawSocket::new(IpVersion::Ipv4, IpProtocol::Unknown(IP_PROTO),
-                           rx_buffer, tx_buffer)
+        pub fn socket(
+            rx_buffer: RawSocketBuffer<'static>,
+            tx_buffer: RawSocketBuffer<'static>,
+        ) -> RawSocket<'static> {
+            RawSocket::new(
+                IpVersion::Ipv4,
+                IpProtocol::Unknown(IP_PROTO),
+                rx_buffer,
+                tx_buffer,
+            )
         }
 
         pub const IP_PROTO: u8 = 63;
@@ -332,60 +360,54 @@ mod test {
             dst_addr: Ipv4Address([10, 0, 0, 2]),
             protocol: IpProtocol::Unknown(IP_PROTO),
             payload_len: 4,
-            hop_limit: 64
+            hop_limit: 64,
         });
         pub const PACKET_BYTES: [u8; 24] = [
-            0x45, 0x00, 0x00, 0x18,
-            0x00, 0x00, 0x40, 0x00,
-            0x40, 0x3f, 0x00, 0x00,
-            0x0a, 0x00, 0x00, 0x01,
-            0x0a, 0x00, 0x00, 0x02,
-            0xaa, 0x00, 0x00, 0xff
-        ];
-        pub const PACKET_PAYLOAD: [u8; 4] = [
-            0xaa, 0x00, 0x00, 0xff
+            0x45, 0x00, 0x00, 0x18, 0x00, 0x00, 0x40, 0x00, 0x40, 0x3f, 0x00, 0x00, 0x0a, 0x00,
+            0x00, 0x01, 0x0a, 0x00, 0x00, 0x02, 0xaa, 0x00, 0x00, 0xff,
         ];
+        pub const PACKET_PAYLOAD: [u8; 4] = [0xaa, 0x00, 0x00, 0xff];
     }
 
     #[cfg(feature = "proto-ipv6")]
     mod ipv6_locals {
         use super::*;
 
-        pub fn socket(rx_buffer: RawSocketBuffer<'static>,
-                      tx_buffer: RawSocketBuffer<'static>)
-                     -> RawSocket<'static> {
-            RawSocket::new(IpVersion::Ipv6, IpProtocol::Unknown(IP_PROTO),
-                           rx_buffer, tx_buffer)
+        pub fn socket(
+            rx_buffer: RawSocketBuffer<'static>,
+            tx_buffer: RawSocketBuffer<'static>,
+        ) -> RawSocket<'static> {
+            RawSocket::new(
+                IpVersion::Ipv6,
+                IpProtocol::Unknown(IP_PROTO),
+                rx_buffer,
+                tx_buffer,
+            )
         }
 
         pub const IP_PROTO: u8 = 63;
         pub const HEADER_REPR: IpRepr = IpRepr::Ipv6(Ipv6Repr {
-            src_addr: Ipv6Address([0xfe, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01]),
-            dst_addr: Ipv6Address([0xfe, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02]),
+            src_addr: Ipv6Address([
+                0xfe, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                0x00, 0x01,
+            ]),
+            dst_addr: Ipv6Address([
+                0xfe, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+                0x00, 0x02,
+            ]),
             next_header: IpProtocol::Unknown(IP_PROTO),
             payload_len: 4,
-            hop_limit: 64
+            hop_limit: 64,
         });
 
         pub const PACKET_BYTES: [u8; 44] = [
-            0x60, 0x00, 0x00, 0x00,
-            0x00, 0x04, 0x3f, 0x40,
-            0xfe, 0x80, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x01,
-            0xfe, 0x80, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x00,
-            0x00, 0x00, 0x00, 0x02,
-            0xaa, 0x00, 0x00, 0xff
+            0x60, 0x00, 0x00, 0x00, 0x00, 0x04, 0x3f, 0x40, 0xfe, 0x80, 0x00, 0x00, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0xfe, 0x80, 0x00, 0x00,
+            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0xaa, 0x00,
+            0x00, 0xff,
         ];
 
-        pub const PACKET_PAYLOAD: [u8; 4] = [
-            0xaa, 0x00, 0x00, 0xff
-        ];
+        pub const PACKET_PAYLOAD: [u8; 4] = [0xaa, 0x00, 0x00, 0xff];
     }
 
     macro_rules! reusable_ip_specific_tests {
@@ -401,39 +423,47 @@ mod test {
 
                 #[test]
                 fn test_send_dispatch() {
-                    let checksum_caps = &ChecksumCapabilities::default();
                     let mut socket = $socket(buffer(0), buffer(1));
+                    let mut cx = Context::mock();
 
                     assert!(socket.can_send());
-                    assert_eq!(socket.dispatch(&checksum_caps, |_| unreachable!()),
-                               Err(Error::Exhausted));
+                    assert_eq!(
+                        socket.dispatch(&mut cx, |_, _| unreachable!()),
+                        Err(Error::Exhausted)
+                    );
 
                     assert_eq!(socket.send_slice(&$packet[..]), Ok(()));
                     assert_eq!(socket.send_slice(b""), Err(Error::Exhausted));
                     assert!(!socket.can_send());
 
-                    assert_eq!(socket.dispatch(&checksum_caps, |(ip_repr, ip_payload)| {
-                        assert_eq!(ip_repr, $hdr);
-                        assert_eq!(ip_payload, &$payload);
+                    assert_eq!(
+                        socket.dispatch(&mut cx, |_, (ip_repr, ip_payload)| {
+                            assert_eq!(ip_repr, $hdr);
+                            assert_eq!(ip_payload, &$payload);
+                            Err(Error::Unaddressable)
+                        }),
                         Err(Error::Unaddressable)
-                    }), Err(Error::Unaddressable));
+                    );
                     assert!(!socket.can_send());
 
-                    assert_eq!(socket.dispatch(&checksum_caps, |(ip_repr, ip_payload)| {
-                        assert_eq!(ip_repr, $hdr);
-                        assert_eq!(ip_payload, &$payload);
+                    assert_eq!(
+                        socket.dispatch(&mut cx, |_, (ip_repr, ip_payload)| {
+                            assert_eq!(ip_repr, $hdr);
+                            assert_eq!(ip_payload, &$payload);
+                            Ok(())
+                        }),
                         Ok(())
-                    }), Ok(()));
+                    );
                     assert!(socket.can_send());
                 }
 
                 #[test]
                 fn test_recv_truncated_slice() {
                     let mut socket = $socket(buffer(1), buffer(0));
+                    let mut cx = Context::mock();
 
                     assert!(socket.accepts(&$hdr));
-                    assert_eq!(socket.process(&$hdr, &$payload,
-                                              &ChecksumCapabilities::default()), Ok(()));
+                    assert_eq!(socket.process(&mut cx, &$hdr, &$payload), Ok(()));
 
                     let mut slice = [0; 4];
                     assert_eq!(socket.recv_slice(&mut slice[..]), Ok(4));
@@ -443,66 +473,75 @@ mod test {
                 #[test]
                 fn test_recv_truncated_packet() {
                     let mut socket = $socket(buffer(1), buffer(0));
+                    let mut cx = Context::mock();
 
                     let mut buffer = vec![0; 128];
                     buffer[..$packet.len()].copy_from_slice(&$packet[..]);
 
                     assert!(socket.accepts(&$hdr));
-                    assert_eq!(socket.process(&$hdr, &buffer, &ChecksumCapabilities::default()),
-                               Err(Error::Truncated));
+                    assert_eq!(
+                        socket.process(&mut cx, &$hdr, &buffer),
+                        Err(Error::Truncated)
+                    );
                 }
             }
-        }
+        };
     }
 
     #[cfg(feature = "proto-ipv4")]
-    reusable_ip_specific_tests!(ipv4, ipv4_locals::socket, ipv4_locals::HEADER_REPR,
-                                ipv4_locals::PACKET_BYTES, ipv4_locals::PACKET_PAYLOAD);
+    reusable_ip_specific_tests!(
+        ipv4,
+        ipv4_locals::socket,
+        ipv4_locals::HEADER_REPR,
+        ipv4_locals::PACKET_BYTES,
+        ipv4_locals::PACKET_PAYLOAD
+    );
 
     #[cfg(feature = "proto-ipv6")]
-    reusable_ip_specific_tests!(ipv6, ipv6_locals::socket, ipv6_locals::HEADER_REPR,
-                                ipv6_locals::PACKET_BYTES, ipv6_locals::PACKET_PAYLOAD);
-
+    reusable_ip_specific_tests!(
+        ipv6,
+        ipv6_locals::socket,
+        ipv6_locals::HEADER_REPR,
+        ipv6_locals::PACKET_BYTES,
+        ipv6_locals::PACKET_PAYLOAD
+    );
 
     #[test]
     #[cfg(feature = "proto-ipv4")]
     fn test_send_illegal() {
-        let checksum_caps = &ChecksumCapabilities::default();
         #[cfg(feature = "proto-ipv4")]
         {
             let mut socket = ipv4_locals::socket(buffer(0), buffer(2));
+            let mut cx = Context::mock();
 
             let mut wrong_version = ipv4_locals::PACKET_BYTES;
             Ipv4Packet::new_unchecked(&mut wrong_version).set_version(6);
 
             assert_eq!(socket.send_slice(&wrong_version[..]), Ok(()));
-            assert_eq!(socket.dispatch(&checksum_caps, |_| unreachable!()),
-                       Ok(()));
+            assert_eq!(socket.dispatch(&mut cx, |_, _| unreachable!()), Ok(()));
 
             let mut wrong_protocol = ipv4_locals::PACKET_BYTES;
             Ipv4Packet::new_unchecked(&mut wrong_protocol).set_protocol(IpProtocol::Tcp);
 
             assert_eq!(socket.send_slice(&wrong_protocol[..]), Ok(()));
-            assert_eq!(socket.dispatch(&checksum_caps, |_| unreachable!()),
-                       Ok(()));
+            assert_eq!(socket.dispatch(&mut cx, |_, _| unreachable!()), Ok(()));
         }
         #[cfg(feature = "proto-ipv6")]
         {
             let mut socket = ipv6_locals::socket(buffer(0), buffer(2));
+            let mut cx = Context::mock();
 
             let mut wrong_version = ipv6_locals::PACKET_BYTES;
             Ipv6Packet::new_unchecked(&mut wrong_version[..]).set_version(4);
 
             assert_eq!(socket.send_slice(&wrong_version[..]), Ok(()));
-            assert_eq!(socket.dispatch(&checksum_caps, |_| unreachable!()),
-                       Ok(()));
+            assert_eq!(socket.dispatch(&mut cx, |_, _| unreachable!()), Ok(()));
 
             let mut wrong_protocol = ipv6_locals::PACKET_BYTES;
             Ipv6Packet::new_unchecked(&mut wrong_protocol[..]).set_next_header(IpProtocol::Tcp);
 
             assert_eq!(socket.send_slice(&wrong_protocol[..]), Ok(()));
-            assert_eq!(socket.dispatch(&checksum_caps, |_| unreachable!()),
-                       Ok(()));
+            assert_eq!(socket.dispatch(&mut cx, |_, _| unreachable!()), Ok(()));
         }
     }
 
@@ -512,21 +551,32 @@ mod test {
         {
             let mut socket = ipv4_locals::socket(buffer(1), buffer(0));
             assert!(!socket.can_recv());
+            let mut cx = Context::mock();
 
             let mut cksumd_packet = ipv4_locals::PACKET_BYTES;
             Ipv4Packet::new_unchecked(&mut cksumd_packet).fill_checksum();
 
             assert_eq!(socket.recv(), Err(Error::Exhausted));
             assert!(socket.accepts(&ipv4_locals::HEADER_REPR));
-            assert_eq!(socket.process(&ipv4_locals::HEADER_REPR, &ipv4_locals::PACKET_PAYLOAD,
-                                      &ChecksumCapabilities::default()),
-                       Ok(()));
+            assert_eq!(
+                socket.process(
+                    &mut cx,
+                    &ipv4_locals::HEADER_REPR,
+                    &ipv4_locals::PACKET_PAYLOAD
+                ),
+                Ok(())
+            );
             assert!(socket.can_recv());
 
             assert!(socket.accepts(&ipv4_locals::HEADER_REPR));
-            assert_eq!(socket.process(&ipv4_locals::HEADER_REPR, &ipv4_locals::PACKET_PAYLOAD,
-                                      &ChecksumCapabilities::default()),
-                       Err(Error::Exhausted));
+            assert_eq!(
+                socket.process(
+                    &mut cx,
+                    &ipv4_locals::HEADER_REPR,
+                    &ipv4_locals::PACKET_PAYLOAD
+                ),
+                Err(Error::Exhausted)
+            );
             assert_eq!(socket.recv(), Ok(&cksumd_packet[..]));
             assert!(!socket.can_recv());
         }
@@ -534,18 +584,29 @@ mod test {
         {
             let mut socket = ipv6_locals::socket(buffer(1), buffer(0));
             assert!(!socket.can_recv());
+            let mut cx = Context::mock();
 
             assert_eq!(socket.recv(), Err(Error::Exhausted));
             assert!(socket.accepts(&ipv6_locals::HEADER_REPR));
-            assert_eq!(socket.process(&ipv6_locals::HEADER_REPR, &ipv6_locals::PACKET_PAYLOAD,
-                                      &ChecksumCapabilities::default()),
-                       Ok(()));
+            assert_eq!(
+                socket.process(
+                    &mut cx,
+                    &ipv6_locals::HEADER_REPR,
+                    &ipv6_locals::PACKET_PAYLOAD
+                ),
+                Ok(())
+            );
             assert!(socket.can_recv());
 
             assert!(socket.accepts(&ipv6_locals::HEADER_REPR));
-            assert_eq!(socket.process(&ipv6_locals::HEADER_REPR, &ipv6_locals::PACKET_PAYLOAD,
-                                      &ChecksumCapabilities::default()),
-                       Err(Error::Exhausted));
+            assert_eq!(
+                socket.process(
+                    &mut cx,
+                    &ipv6_locals::HEADER_REPR,
+                    &ipv6_locals::PACKET_PAYLOAD
+                ),
+                Err(Error::Exhausted)
+            );
             assert_eq!(socket.recv(), Ok(&ipv6_locals::PACKET_BYTES[..]));
             assert!(!socket.can_recv());
         }
@@ -555,16 +616,24 @@ mod test {
     fn test_doesnt_accept_wrong_proto() {
         #[cfg(feature = "proto-ipv4")]
         {
-            let socket = RawSocket::new(IpVersion::Ipv4,
-                IpProtocol::Unknown(ipv4_locals::IP_PROTO+1), buffer(1), buffer(1));
+            let socket = RawSocket::new(
+                IpVersion::Ipv4,
+                IpProtocol::Unknown(ipv4_locals::IP_PROTO + 1),
+                buffer(1),
+                buffer(1),
+            );
             assert!(!socket.accepts(&ipv4_locals::HEADER_REPR));
             #[cfg(feature = "proto-ipv6")]
             assert!(!socket.accepts(&ipv6_locals::HEADER_REPR));
         }
         #[cfg(feature = "proto-ipv6")]
         {
-            let socket = RawSocket::new(IpVersion::Ipv6,
-                IpProtocol::Unknown(ipv6_locals::IP_PROTO+1), buffer(1), buffer(1));
+            let socket = RawSocket::new(
+                IpVersion::Ipv6,
+                IpProtocol::Unknown(ipv6_locals::IP_PROTO + 1),
+                buffer(1),
+                buffer(1),
+            );
             assert!(!socket.accepts(&ipv6_locals::HEADER_REPR));
             #[cfg(feature = "proto-ipv4")]
             assert!(!socket.accepts(&ipv4_locals::HEADER_REPR));
